# This file is part of GNU Enterprise.
#
# GNU Enterprise is free software; you can redistribute it
# and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# GNU Enterprise is distributed in the hope that it will be
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with program; see the file COPYING. If not,
# write to the Free Software Foundation, Inc., 59 Temple Place
# - Suite 330, Boston, MA 02111-1307, USA.
#
# Copyright 2000-2005 Free Software Foundation
#
# $Id: entry.py 7979 2005-09-23 13:39:47Z btami $
#
# DESCRIPTION:
# A PyWin32 based user interface driver for GNUe forms.
#
# NOTES:
#

import win32gui
import win32con

from gnue.common import events
from gnue.forms.uidrivers.win32.widgets._base import UIHelper, Win32Entry
from gnue.forms.uidrivers.win32.common import *

#
# UIEntry
#
# Widget set specific function that creates a single instance of a data entry widget
#
class UIEntry(UIHelper):
  def _createWidget(self, event, spacer):
    object = event.object

    ostyle = object.style
    height = self.itemHeight
    width = self.itemWidth
    title = ""
    
    if ostyle == 'dropdown':
      #~ style = win32con.WS_CHILD | win32con.WS_VSCROLL | win32con.LBS_NOTIFY | \
                 #~ win32con.CBS_DROPDOWNLIST | win32con.CBS_NOINTEGRALHEIGHT
      style = win32con.WS_CHILD | win32con.WS_VSCROLL | \
                 win32con.CBS_DROPDOWN | win32con.CBS_NOINTEGRALHEIGHT
      styleEx = 0
      className = "COMBOBOX"
      height = self.itemHeight +100 # TODO

    elif ostyle == 'listbox':
      style = win32con.WS_CHILD | win32con.WS_VSCROLL | win32con.LBS_NOTIFY
      styleEx = win32con.WS_EX_CLIENTEDGE
      className = "LISTBOX"

    elif ostyle == 'label':
      style = win32con.WS_CHILD
      styleEx = 0
      className = "STATIC"

    elif ostyle == 'checkbox':
      style = win32con.WS_CHILD | win32con.BS_AUTOCHECKBOX
      styleEx = 0
      className = "BUTTON"
      title = object.label
      
    else: # Normal text box
      height = self.itemHeight + 4
      width = self.itemWidth
      style = win32con.WS_CHILD | win32con.ES_NOHIDESEL #| win32con.WS_BORDER

      if hasattr(object,'Char__height') and object.Char__height > 1:
        style = style | win32con.ES_MULTILINE | win32con.ES_AUTOVSCROLL | \
                win32con.WS_VSCROLL #| win32con.ES_WANTRETURN
        width = self.itemWidth

      styleEx = win32con.WS_EX_CLIENTEDGE
      className = "EDIT"

    newWidget = Win32Entry(self._uiDriver, styleEx, className, title, style,
                        self.itemX,
                        self.itemY, 
                        width,
                        height,
                        event.container, getNextId())

    if ostyle == 'dropdown' or ostyle == 'listbox':
      if event.initialize:
        choices = object._field.allowedValues()[1]
      else:
        choices = [""]

      if ostyle == 'dropdown':
        ADDSTRING = win32con.CB_ADDSTRING
      else:
        ADDSTRING = win32con.LB_ADDSTRING

      for item in choices:
        win32gui.SendMessage(newWidget.GetHwnd(), ADDSTRING, 0, textEncode(item))
      if event.initialize:
        newWidget._origAllowedValues = object._field._allowedValues
        newWidget.SetValue("")

    return newWidget


configuration = {
    'baseClass'  : UIEntry,
    'provides'   : 'GFEntry',
    'container'  : 0,
  }
